@@ -19,11 +19,13 @@ type solver
     for universe consistency (cf. Cudf_checker.is_consistent) *)
 val load : ?check : bool -> Cudf.universe -> solver
 
+(*
 (** Turn a result from Diagnostic_int into one of Diagnostic *)
-val result : Depsolver_int.identity -> Cudf.universe -> Diagnostic_int.result -> Diagnostic.result
+val result : Depsolver_int.identity -> Cudf.universe -> Diagnostic.result_int -> Diagnostic.result
 
 (** Turn a request from Diagnostic_int into one of Diagnostic *)
-val request : Cudf.universe -> Diagnostic_int.request -> Diagnostic.request
+val request : Cudf.universe -> Diagnostic.request_int -> Diagnostic.request
+*)
 
 (** check if the given package can be installed in the universe 
  
@@ -95,7 +97,6 @@ val univcheck : ?global_constraints:bool -> ?callback:(Diagnostic.diagnosis -> u
    @param global_constraints : enforce global constraints on the given universe.
    @return the number of broken packages
  *)
-
 val listcheck : ?global_constraints:bool -> ?callback:(Diagnostic.diagnosis -> unit) -> 
   Cudf.universe -> Cudf.package list -> int 
 
@@ -166,3 +167,11 @@ val check_request_using:
   ?criteria:string ->
   ?explain : bool ->
   Cudf.cudf -> solver_result
+
+#ifdef HASOCAMLGRAPH
+(** Build the installation graph from a cudf solution universe and sets of packages to be 
+    installed/removed (see CudfAdd.make_summary) *)
+val installation_graph: solution:Cudf.universe -> 
+  (Common.CudfAdd.Cudf_set.t * Common.CudfAdd.Cudf_set.t) -> 
+    Defaultgraphs.ActionGraph.G.t
+#endif
