@@ -31,16 +31,15 @@ module Options = struct
   let options = OptParser.make ~description
   include StdOptions.MakeOptions(struct let options = options end)
 
-  exception Format
   let otypes = ["cnf";"dimacs";"cudf";"deb";"debsrc";"dot";"gml";"grml";"table"]
   let out_option ?default ?(metavar = Printf.sprintf "<%s>" (String.concat "|" otypes)) () =
-    let corce s = if List.mem s otypes then s else raise Format in
+    let corce s = if List.mem s otypes then s else raise Not_found in
     let error _ s = Printf.sprintf "%s format not supported" s in
     Opt.value_option metavar default corce error
 
   let gtypes = ["syn";"pkg";"conj";"strdeps";"strcnf";"dom"]
   let grp_option ?default ?(metavar = Printf.sprintf "<%s>" (String.concat "|" gtypes)) () =
-    let corce s = if List.mem s gtypes then s else raise Format in
+    let corce s = if List.mem s gtypes then s else raise Not_found in
     let error _ s = Printf.sprintf "%s format not supported" s in
     Opt.value_option metavar default corce error
 
@@ -66,7 +65,8 @@ module Options = struct
   StdOptions.OutputOptions.add_option options ~short_name:'T' ~help:"Output type format. Default cnf" out_type;;
 
   include StdOptions.DistribOptions ;;
-  StdOptions.DistribOptions.add_options options ;;
+  StdOptions.DistribOptions.add_debian_options options ;;
+  StdOptions.DistribOptions.add_opam_options options ;;
 
 end;;
 
@@ -88,11 +88,11 @@ let loadl to_cudf l =
 ;;
 
 let parse_request to_cudf l =
-  let open Debian in
   let pkgs_of s = (* convert request into list of packages *) 
     let rs = String.strip (snd (String.split s " ")) in
-    let  f = Packages.lexbuf_wrapper Packages_parser.vpkglist_top in
-    loadl to_cudf (f (Format822.dummy_loc,rs)) 
+    let field = ("ceve pkg req",(Format822.dummy_loc,rs)) in
+    let v = Pef.Packages.lexbuf_wrapper Pef.Packages_parser.vpkglist_top field in
+    loadl to_cudf v 
   in              
   let parse acc s =
     if String.starts_with s "install: " then
@@ -231,21 +231,19 @@ let main () =
       |"dimacs" -> Printf.fprintf oc "%s" (Depsolver.output_clauses ~global_constraints ~enc:Depsolver.Dimacs u)
       |"cudf" -> Cudf_printer.pp_cudf oc doc
       |"deb" -> List.iter (function
-          | StdLoaders.Deb p ->
-            Printf.fprintf oc "%a\n" Debian.Printer.pp_package p
+          | StdLoaders.Deb p -> p#pp oc
           | StdLoaders.DebSrc p -> ()
           | _ -> failwith "impossible"
         ) (List.map cudf2deb l)
       |"debsrc" -> List.iter (function
           | StdLoaders.Deb p -> ()
-          | StdLoaders.DebSrc p ->
-            Printf.fprintf oc "%a\n" Debian.Printer.pp_source p
+          | StdLoaders.DebSrc p -> p#pp oc
           | _ -> failwith "impossible"
         ) (List.map cudf2deb l)
       |"table" ->
 #ifdef HASOCAMLGRAPH 
         Printf.fprintf oc "%d\t%d\t%d\n"
-        (Cudf.universe_size u) (DGraph.G.nb_edges (DGraph.dependency_graph u))
+        (Cudf.universe_size u) (Defaultgraphs.SyntacticDependencyGraph.G.nb_edges (Defaultgraphs.SyntacticDependencyGraph.dependency_graph u))
         (nr_conflicts u)
 #else
         failwith (Printf.sprintf "format table not supported: needs ocamlgraph")
@@ -256,14 +254,14 @@ let main () =
         let fmt = Format.formatter_of_out_channel oc in
         begin match OptParse.Opt.get Options.grp_type with
           |"syn" ->
-            let g = DGraph.dependency_graph u in
-            if t = "dot" then DGraph.DotPrinter.print fmt g
-            else if t = "gml" then DGraph.GmlPrinter.print fmt g
-            else if t = "grml" then DGraph.GraphmlPrinter.print fmt g
+            let g = Defaultgraphs.SyntacticDependencyGraph.dependency_graph u in
+            if t = "dot" then Defaultgraphs.SyntacticDependencyGraph.DotPrinter.print fmt g
+            else if t = "gml" then Defaultgraphs.SyntacticDependencyGraph.GmlPrinter.print fmt g
+            else if t = "grml" then Defaultgraphs.SyntacticDependencyGraph.GraphmlPrinter.print fmt g
             else assert false
           |("pkg" | "strdeps" | "conj"| "dom") as gt ->
             let g =
-              if gt = "pkg" then PGraph.dependency_graph u
+              if gt = "pkg" then Defaultgraphs.PackageGraph.dependency_graph u
               else if gt = "strdeps" then
                 let g = Strongdeps.strongdeps_univ u in
                 (* if input are Debian packages and --deb-ignore-essential was
@@ -287,11 +285,11 @@ let main () =
               else assert false
             in
             if t = "dot" then
-              PGraph.DotPrinter.print fmt g
+              Defaultgraphs.PackageGraph.DotPrinter.print fmt g
             else if t = "gml" then
-              PGraph.GmlPrinter.print fmt g
+              Defaultgraphs.PackageGraph.GmlPrinter.print fmt g
             else if t = "grml" then
-              PGraph.GraphmlPrinter.print fmt g
+              Defaultgraphs.PackageGraph.GraphmlPrinter.print fmt g
             else assert false
           |s -> failwith (Printf.sprintf "type %s not supported" s)
         end
