@@ -21,16 +21,21 @@ include Util.Logging(struct let label = label end) ;;
 
 type debtypes = [ `Edsp | `Deb | `DebSrc ]
 type rpmtypes = [ `Synthesis | `Hdlist ]
-type othertypes = [ `Csw | `Pef ]
+type othertypes = [ `Csw | `Pef | `Opam ]
 
 type filetypes = [ `Cudf | debtypes | rpmtypes | othertypes ]
 
+let supported_input_types =
+  [`Edsp; `Deb ; `DebSrc ; `Synthesis ; `Hdlist ; `Pef ; `Opam ; `Csw ; `Cudf ]
+;;
+
 let scheme_to_string = function
   | `Edsp -> "edsp"
   | `Csw -> "csw"
   | `Deb -> "deb"
   | `DebSrc -> "debsrc"
   | `Pef -> "pef"
+  | `Opam -> "opam"
   | `Cudf -> "cudf"
   | `Synthesis -> "synthesis"
   | `Hdlist -> "hdlist"
@@ -40,16 +45,15 @@ let scheme_of_string = function
   | "edsp" -> `Edsp
   | "csw" -> `Csw
   | "deb" -> `Deb
+  | "opam" -> `Opam
   | "debsrc" -> `DebSrc
   | "cudf" -> `Cudf
   | "eclipse" | "pef" -> `Pef
   | "synthesis" -> `Synthesis
   | "hdlist" -> `Hdlist
-  | s -> fatal "unknown input scheme" s
-;;
-
-let supported_input_types =
-  [`Edsp; `Deb ; `DebSrc ; `Synthesis ; `Hdlist ; `Pef ; `Csw ; `Cudf ]
+  | s ->
+    let supported = String.concat ", " (List.map scheme_to_string supported_input_types) in
+    fatal "unknown input scheme: \"%s\" - Must be one of: %s" s supported
 ;;
 
 (***********************************************************************)
