@@ -12,7 +12,11 @@
 
 (** Debian Specific Cudf conversion routines *)
 
-module SSet = Set.Make(String)
+module SSet = Set.Make(struct
+  type t = string * Pef.Packages_types.constr option
+  let compare = Pervasives.compare
+end)
+module OcamlHashtbl = Hashtbl
 
 open ExtLib
 open Common
@@ -23,10 +27,12 @@ module Version = Versioning.Debian
 let label =  __label ;;
 include Util.Logging(struct let label = label end) ;;
 
+let max32int = Int32.to_int(Int32.max_int);;
+
 module SMap = Map.Make (String)
 
 type tables = {
-  virtual_table : SSet.t ref Util.StringHashtbl.t; (** all names of virtual packages *)
+  virtual_table : (string,SSet.t ref) OcamlHashtbl.t; (** all names of virtual packages *)
   unit_table : unit Util.StringHashtbl.t ;         (** all names of real packages    *)
   versions_table : int Util.StringHashtbl.t;       (** version -> int table          *)
   reverse_table : (string SMap.t) ref Util.IntHashtbl.t;
@@ -36,7 +42,7 @@ type tables = {
 }
 
 let create n = {
-  virtual_table = Util.StringHashtbl.create (10 * n);
+  virtual_table = OcamlHashtbl.create (10 * n);
   unit_table = Util.StringHashtbl.create (2 * n);
   versions_table = Util.StringHashtbl.create (10 * n);
   versioned_table = Util.StringHashtbl.create (10 *n);
@@ -68,13 +74,15 @@ let default_options = {
   builds_from = false;
 }
 
-let add_name_arch n a = CudfAdd.encode (Printf.sprintf "%s:%s" n a)
+let add_name_arch n = function
+  |"" -> CudfAdd.encode n
+  |a -> CudfAdd.encode (Printf.sprintf "%s:%s" n a) 
+
 let add_arch native_arch name = function
   |"all" -> add_name_arch name native_arch
   |package_arch -> add_name_arch name package_arch
-;;
 
-(** add arch info to a vpkg 
+(* add arch info to a vpkg 
  - if it's a :any dependency then just encode the name without arch information :
    means that this dependency/conflict can be satified by any packages
  - if it is a :native dependency then add the native architecture
@@ -84,27 +92,16 @@ let add_arch native_arch name = function
      dependencies on native architecture packages.
    + otherwise all dependencies are satisfied by packages of the same architecture
      of the package we are considering 
-Pcre.regexp "^(.* )(:(any|native|.* )){1}$" ;;
 *)
-let add_arch_deps native_arch package_arch name = 
-  try 
-    match String.split name ":" with
-    |n,"any" -> CudfAdd.encode n
-    |n,"native" -> add_name_arch n native_arch
-    |n,a -> add_name_arch n a
-  with ExtString.Invalid_string ->
-    add_arch native_arch name package_arch
-;;
-
-let add_arch_l ?(deps=false) native_arch package_arch l = 
-  if deps then
-    List.map (fun (n,c) -> ((add_arch_deps native_arch package_arch n),c)) l
-  else
-    List.map (fun (n,c) -> ((add_arch native_arch n package_arch),c)) l
+let add_arch_info ?(native_arch="") ?(package_arch="") = function
+  |n,Some "any" -> CudfAdd.encode n
+  |n,Some "native" -> add_name_arch n native_arch
+  |n,Some a -> add_name_arch n a
+  |n,None -> add_arch native_arch n package_arch
 ;;
 
 let clear tables =
-  Util.StringHashtbl.clear tables.virtual_table;
+  OcamlHashtbl.clear tables.virtual_table;
   Util.StringHashtbl.clear tables.unit_table;
   Util.StringHashtbl.clear tables.versions_table;
   Util.StringHashtbl.clear tables.versioned_table;
@@ -120,27 +117,34 @@ let add_v table k v =
     Util.StringPairHashtbl.add table k v
 
 let add_s h k v =
-  try let s = Util.StringHashtbl.find h k in s := SSet.add v !s
-  with Not_found -> Util.StringHashtbl.add h k (ref (SSet.singleton v))
+  try let s = OcamlHashtbl.find h k in s := SSet.add v !s
+  with Not_found -> OcamlHashtbl.add h k (ref (SSet.singleton v))
 ;;
 
 (* collect names of virtual packages 
  * associates to each virtual package a list of real packages *)
 let init_virtual_table table pkg =
-  List.iter (fun ((name,_),_) -> add_s table name pkg.name) pkg.provides
+  List.iter (fun ((name,_),constr) -> 
+    add_s table name (pkg#name,constr)
+    (*
+    match CudfAdd.cudfop constr with
+    |None -> add_s table name (pkg#name,None)
+    |Some(_,version) as c -> add_s table name (pkg#name,c)
+  *)
+  ) pkg#provides
 
 (* collect names of real packages *)
 let init_unit_table table pkg = 
-  add table pkg.name ()
+  add table pkg#name ()
 
 (* collect all versions mentioned of depends, pre_depends, conflict and breaks *)
 let init_versioned_table table pkg =
   let conj_iter l = List.iter (fun ((name,_),_)-> add table name ()) l in
   let cnf_iter ll = List.iter conj_iter ll in
-  conj_iter pkg.conflicts ;
-  conj_iter pkg.breaks ;
-  cnf_iter pkg.pre_depends ;
-  cnf_iter pkg.depends
+  conj_iter pkg#conflicts ;
+  conj_iter pkg#breaks ;
+  cnf_iter pkg#pre_depends ;
+  cnf_iter pkg#depends
 ;;
 
 (* collect all versions mentioned anywhere in the universe, including source fields *)
@@ -157,15 +161,15 @@ let init_versions_table table pkg =
     |(p,None) -> add_v table (pv,p) ()
     |(p,Some(v)) -> add_v table (v,p) ()
   in 
-  add_v table (pkg.version,pkg.name) ();
-  conj_iter pkg.breaks;
-  conj_iter pkg.provides;
-  conj_iter pkg.conflicts ;
-  conj_iter pkg.replaces;
-  cnf_iter pkg.depends;
-  cnf_iter pkg.pre_depends;
-  cnf_iter pkg.recommends;
-  add_source pkg.version pkg.source
+  add_v table (pkg#version,pkg#name) ();
+  conj_iter pkg#breaks;
+  conj_iter pkg#provides;
+  conj_iter pkg#conflicts ;
+  conj_iter pkg#replaces;
+  cnf_iter pkg#depends;
+  cnf_iter pkg#pre_depends;
+  cnf_iter pkg#recommends;
+  add_source pkg#version pkg#source
 ;;
 
 let init_tables ?(step=1) ?(versionlist=[]) pkglist =
@@ -218,81 +222,94 @@ let get_cudf_version tables (package,version) =
   end
 
 let get_real_version tables (name,cudfversion) =
-  let package = 
-    (* XXX this is a hack. I should record the name with the architecture *)
-    let n = CudfAdd.decode name in
-    try
-      let (n,a) = ExtString.String.split n ":" in
-      if n = "src" then a else n
-    with Invalid_string -> n
+  let package =
+    (* XXX this is a hack. Remove --virtual- and architecture encoding *)
+    let n = (CudfAdd.decode name) in
+    if ExtString.String.starts_with n "--vir" then
+      ExtString.String.slice ~first:10 n
+    else
+      try
+        let (n,a) = ExtString.String.split n ":" in
+        if n = "src" then a else n
+      with Invalid_string -> n
   in
   try
-    let m = !(Util.IntHashtbl.find tables.reverse_table cudfversion) in
-    try SMap.find package m 
-    with Not_found ->
-      let known =
-        String.concat "," (
-          List.map (fun (n,v) ->
-            Printf.sprintf "(%s,%s)" n v
-          ) (SMap.bindings m)
-        )
-      in
-      fatal "Unable to get real version for %s (%i)\n All Known versions for this package are %s" package cudfversion known 
+    if cudfversion = max32int || cudfversion = max32int - 1 then 
+      (package,"nan")
+    else
+      let m = !(Util.IntHashtbl.find tables.reverse_table cudfversion) in
+      try (package,SMap.find package m) 
+      with Not_found ->
+        let known =
+          String.concat "," (
+            List.map (fun (n,v) ->
+              Printf.sprintf "(%s,%s)" n v
+            ) (SMap.bindings m)
+          )
+        in
+        fatal "Unable to get real version for %s (%i)\n All Known versions for this package are %s" package cudfversion known 
   with Not_found ->
     fatal "Package (%s,%d) does not have an associated debian version" name cudfversion
 
-let loadl ?(enc=false) tables l =
+let loadl ?native_arch ?package_arch tables l =
   List.flatten (
-    List.map (fun ((name,aop),constr) ->
-      let encname = 
-        let n = match aop with Some a -> name^":"^a | None -> name in
-        if enc then CudfAdd.encode n else n
-      in
+    List.map (fun ((name,_) as vpkgname,constr) ->
+      let encname = add_arch_info ?native_arch ?package_arch vpkgname in
       match CudfAdd.cudfop constr with
       |None ->
-          if (Util.StringHashtbl.mem tables.virtual_table name) &&
-          (Util.StringHashtbl.mem tables.versioned_table name) then
-            [(encname, None);("--virtual-"^encname, None)]
+          (* Versioned virtual packages will satisfiy non versioned dependencies *)
+          if (OcamlHashtbl.mem tables.virtual_table name) then
+            [(encname, None);("--virtual-"^encname,Some(`Eq,max32int - 1))]
           else
             [(encname, None)]
       |Some(op,v) ->
-          (* debian policy 7.5 . If a relationship field has 
-           * a version number attached, only real packages will 
-           * be considered to see whether the relationship is satisfied *)
-          if (Util.StringHashtbl.mem tables.virtual_table name) && 
-             not (Util.StringHashtbl.mem tables.versioned_table name) then []
-          else begin
-            debug "Conflict %s < %s %s" encname name v;
-            [(encname,Some(op,get_cudf_version tables (name,v)))]
-          end
+          (* Non-versioned virtual packages will not satisfy versioned dependencies. *)
+          try
+            match SSet.elements !(OcamlHashtbl.find tables.virtual_table name) with
+            |[] -> assert false
+            |l ->
+                let dl = 
+                  List.filter_map (function
+                    |(_,None) -> Some ("--virtual-"^encname,Some(`Eq,max32int))
+                    |(_,Some _) ->
+                        let constr = Some(op,get_cudf_version tables (name,v)) in
+                        Some ("--virtual-"^encname,constr)
+                  ) l
+                in
+                if Util.StringHashtbl.mem tables.unit_table name then
+                  let constr = Some(op,get_cudf_version tables (name,v)) in
+                  (encname,constr)::dl
+                else dl
+          with Not_found -> 
+              let constr = Some(op,get_cudf_version tables (name,v)) in
+              [(encname,constr)]
     ) l
   )
 
-let loadll ?(enc=false) tables ll =
-  List.filter_map (fun l -> match loadl ~enc tables l with [] -> None |l -> Some l) ll
+let loadll ?native_arch ?package_arch tables ll =
+  List.filter_map (fun l ->
+    match loadl ?native_arch ?package_arch tables l with
+    |[] -> None
+    |l -> Some l
+  ) ll
 
 (* we add a self conflict here, because in debian each package is in conflict
    with all other versions of the same package *)
-let loadlc ?(enc=false) tables name l =
-  let sc = if enc then (CudfAdd.encode name, None) else (name, None) in
-  sc::(loadl ~enc tables l)
+let loadlc ?native_arch ?package_arch tables name l =
+  (CudfAdd.encode name, None)::(loadl ?native_arch ?package_arch tables l)
 
-let loadlp ?(enc=false) tables l =
-  List.map (fun ((name,_),constr) ->
-    let encname = if enc then CudfAdd.encode name else name in
-    match CudfAdd.cudfop constr with
-    |None  ->
-        if (Util.StringHashtbl.mem tables.unit_table name) || 
-        (Util.StringHashtbl.mem tables.versioned_table name)
-        then ("--virtual-"^encname,None)
-        else (encname, None)
-    |Some(`Eq,v) ->
-        if (Util.StringHashtbl.mem tables.unit_table name) || 
-        (Util.StringHashtbl.mem tables.versioned_table name)
-        then ("--virtual-"^encname,Some(`Eq,get_cudf_version tables (name,v)))
-        else (encname,Some(`Eq,get_cudf_version tables (name,v)))
-    |_ -> fatal "This should never happen : a provide can be either = or unversioned"
-  ) l
+let loadlp ?native_arch ?package_arch tables l =
+  List.flatten (
+    List.map (fun ((name,_) as vpkgname,constr) ->
+      let encname = add_arch_info ?native_arch ?package_arch vpkgname in
+      match CudfAdd.cudfop constr with
+      |None -> [("--virtual-"^encname,Some(`Eq,max32int - 1))]
+      |Some(`Eq,v) ->
+        let constr = Some(`Eq,get_cudf_version tables (name,v)) in
+        [("--virtual-"^encname,constr);(encname,constr)]
+      |_ -> fatal "This should never happen : a provide can be either = or unversioned"
+    ) l
+  )
 
 
 (* ========================================= *)
@@ -316,34 +333,42 @@ let preamble =
     ("filename",(`String (Some "")));
     ("installedsize",(`Int (Some 0)));
     ("multiarch",(`String (Some "")));
+    ("native",(`Int (Some 0)));
     ]
   in
   CudfAdd.add_properties Cudf.default_preamble l
 
-let add_extra_default extras tables pkg =
-  let check = function [] :: _ -> failwith (Printf.sprintf "Malformed dep (%s %s)" pkg.name pkg.version) |l -> l in
-  let name = if String.starts_with pkg.name "src:" then
-      String.sub pkg.name 4 ((String.length pkg.name) - 4)
-    else pkg.name
+let add_extra ?native_arch extras tables pkg =
+  let name = 
+    if String.starts_with pkg#name "src:" then
+      String.sub pkg#name 4 ((String.length pkg#name) - 4)
+    else pkg#name
   in
   let name = ("name",`String name) in
-  let number = ("number",`String pkg.version) in
-  let architecture = ("architecture",`String pkg.architecture) in
-  let priority = ("priority",`String pkg.priority) in
-  let essential = ("essential", `Bool pkg.essential) in
-  let build_essential = ("buildessential", `Bool pkg.build_essential) in
+  let number = ("number",`String pkg#version) in
+  let architecture = ("architecture",`String pkg#architecture) in
+  let priority = ("priority",`String pkg#priority) in
+  let essential = ("essential", `Bool pkg#essential) in
+  let build_essential = ("buildessential", `Bool pkg#build_essential) in
   let (source,sourcenumber,sourceversion) =
     let (n,v) =
-      match pkg.source with
-      |("",_) -> (pkg.name,pkg.version)
-      |(n,None) -> (n,pkg.version)
+      match pkg#source with
+      |("",_) -> (pkg#name,pkg#version)
+      |(n,None) -> (n,pkg#version)
       |(n,Some v) -> (n,v)
     in
     let cv = get_cudf_version tables ("",v) in
     ("source",`String n), ("sourcenumber", `String v), ("sourceversion", `Int cv)
   in
-  let recommends = ("recommends", `Vpkgformula (check (loadll ~enc:true tables pkg.recommends))) in
-  let replaces = ("replaces", `Vpkglist (loadl tables ~enc:true pkg.replaces)) in
+  let recommends = ("recommends", `Vpkgformula (loadll tables pkg#recommends)) in
+  let replaces = ("replaces", `Vpkglist (loadl tables pkg#replaces)) in
+  let native = (* XXX if 0 we can ignore it *)
+    let n =
+      if Option.is_some native_arch && 
+        (Option.get native_arch) = pkg#architecture then 1 
+      else 0
+    in ("native",`Int n)
+  in
   let extras = 
     ("Type",("type",`String None))::
     ("Filename",("filename",`String None))::
@@ -354,9 +379,11 @@ let add_extra_default extras tables pkg =
   let l =
     List.filter_map (fun (debprop, (cudfprop,v)) ->
       try 
-        let s = Packages.assoc debprop pkg.extras in
+        let s = pkg#get_extra debprop in
         let typ = Cudf_types.type_of_typedecl v in
-        Some (cudfprop, Cudf_types_pp.parse_value typ s)
+        try Some (cudfprop, Cudf_types_pp.parse_value typ s)
+        with Cudf_types_pp.Type_error _ ->
+          fatal "Cudf Parsing Error while converting properties %s: %s" debprop s
       with Not_found -> None
     ) extras
   in
@@ -369,11 +396,11 @@ let add_extra_default extras tables pkg =
   [priority; name; architecture; number;
   source; sourcenumber; sourceversion; 
   recommends; replaces;
-  essential;build_essential]@ l
+  essential;build_essential;native]@ l
 ;;
 
 let set_keep pkg =
-  match pkg.essential,Packages.is_on_hold pkg with
+  match pkg#essential,Packages.is_on_hold pkg with
   |_,true -> `Keep_version
   |true,_ -> `Keep_package
   |false,_ -> `Keep_none
@@ -383,65 +410,64 @@ let add_inst inst pkg =
   if inst then true 
   else Packages.is_installed pkg
 
-let add_extra extras tables pkg =
-  add_extra_default extras tables pkg
-
 let tocudf tables ?(options=default_options) ?(inst=false) pkg =
   let bind m f = List.flatten (List.map f m) in
-  let encpkgname = CudfAdd.encode pkg.name in
+  let encpkgname = CudfAdd.encode pkg#name in
   if not(Option.is_none options.native) then begin
     let native_arch = Option.get options.native in
-    let pkgarch =
+    let package_arch =
       if Sources.is_source pkg then
         match options.host with
         |None -> native_arch          (* source package : build deps on the native arch *)
         |Some host_arch  -> host_arch (* source package : build deps on the cross arch *)
       else
-        pkg.architecture              (* binary package : dependencies are package specific *)
+        pkg#architecture              (* binary package : dependencies are package specific *)
     in
     let _name = 
       (* if the package is a source package the name does not need an
        * architecture annotation. Nobody depends on it *)
       if Sources.is_source pkg then encpkgname
-      else add_arch native_arch pkg.name pkgarch
+      else add_arch native_arch pkg#name package_arch
     in
-    let _version = get_cudf_version tables (pkg.name,pkg.version)  in
+    let _version = get_cudf_version tables (pkg#name,pkg#version)  in
     let _provides = 
       let archlessprovide = (encpkgname,None) in
       let multiarchprovides = 
-        match pkg.multiarch with
+        match pkg#multiarch with
         |(`No|`Same) ->
-           (* pkgarch provides *)
-             (add_arch_l native_arch pkgarch (loadlp tables pkg.provides))
+           (* package_arch provides *)
+           loadlp ~native_arch ~package_arch tables pkg#provides
         |`Foreign ->
            (* packages of same name and version of itself in all archs except its own
               each package this package provides is provided in all arches *)
            bind (native_arch::options.foreign) (function
-                |arch when arch = pkgarch ->
-                    (add_arch_l native_arch arch (loadlp tables pkg.provides))
+                |arch when arch = package_arch ->
+                    loadlp ~native_arch ~package_arch:arch tables pkg#provides
                 |arch ->
-                    (add_arch native_arch pkg.name arch,Some(`Eq,_version)) ::
-                      (add_arch_l native_arch arch (loadlp tables pkg.provides))
+                    (add_arch native_arch pkg#name arch,Some(`Eq,_version)) ::
+                      (loadlp ~native_arch ~package_arch:arch tables pkg#provides)
               )
         |`Allowed ->
            (* archless package and arch: any package *)
            (* all provides as arch: any *)
-           (* pkgarch provides *)
-           let any = (CudfAdd.encode (pkg.name^":any"),None) in
+           (* package_arch provides *)
+           let any = (CudfAdd.encode (pkg#name^":any"),None) in
            let l = 
-             bind (loadlp tables pkg.provides) (fun (name, c) ->
-               [(CudfAdd.encode (name^":any"), c);
-               ((add_arch native_arch name pkgarch),c)]
+             loadlp ~native_arch ~package_arch tables (
+               bind pkg#provides (fun ((name,_), c) ->
+                   [((name, Some "any"), c);
+                   ((name,None),c)]
+               )
              )
            in any::l
       in archlessprovide :: multiarchprovides
     in
     let _conflicts = 
-      let originalconflicts = pkg.breaks @ pkg.conflicts in
+      let originalconflicts = pkg#breaks @ pkg#conflicts in
       (* self conflict *)
-      let sc = (add_arch native_arch pkg.name pkgarch,None) in
+      let sc = (add_arch native_arch pkg#name package_arch,None) in
       let multiarchconstraints = 
-        match pkg.multiarch with
+        match pkg#multiarch with
         |(`No|`Foreign|`Allowed) -> 
             (* conflict with all other packages with differents archs *)
             let mac = (encpkgname,None) in
@@ -450,22 +476,22 @@ let tocudf tables ?(options=default_options) ?(inst=false) pkg =
             (* conflict with packages of same name but different arch and version*)
             let masc =
               List.filter_map (function
-                |arch when arch = pkgarch -> None
-                |arch -> Some(add_arch native_arch pkg.name arch,Some(`Neq,_version))
+                |arch when arch = package_arch -> None
+                |arch -> Some(add_arch native_arch pkg#name arch,Some(`Neq,_version))
               ) (native_arch::options.foreign)
             in
             sc :: masc 
       in
       let multiarchconflicts =
         let selfconflict = function
-	  | ((n,a),None) -> (n = pkg.name) || List.exists (fun ((p,_),_) -> p=n) pkg.provides 
-	  | ((n,a),_)    -> (n = pkg.name)
+	  | ((n,a),None) -> (n = pkg#name) || List.exists (fun ((p,_),_) -> p=n) pkg#provides 
+	  | ((n,a),_)    -> (n = pkg#name)
 	in
         let realpackage = Util.StringHashtbl.mem tables.unit_table in
-        match pkg.multiarch with
+        match pkg#multiarch with
         |(`No|`Foreign|`Allowed) -> 
             bind (native_arch::options.foreign) (fun arch ->
-                add_arch_l native_arch arch (loadl tables originalconflicts)
+                loadl ~native_arch ~package_arch:arch tables originalconflicts
               )
         |`Same -> 
             (* XXX : Duplicated conflicts ! *)
@@ -485,39 +511,31 @@ let tocudf tables ?(options=default_options) ?(inst=false) pkg =
                       end
                   |false, true ->             (* a virtual package and a self conflict *)
                       begin
-                        debug "M-A-Same: pkg %s has a self-conflict via virtual package: %s" pkg.name n;
+                        debug "M-A-Same: pkg %s has a self-conflict via virtual package: %s" pkg#name n;
                         try
-                          List.filter_map (fun pn ->
-                            if pn <> pkg.name then begin
-                              debug "M-A-Same: adding conflict on real package %s for %s" pn pkg.name; 
+                          List.filter_map (fun (pn,_) ->
+                            if pn <> pkg#name then begin
+                              debug "M-A-Same: adding conflict on real package %s for %s" pn pkg#name; 
                               Some((pn,a),None)
                             end else None
-                          ) (SSet.elements !(Util.StringHashtbl.find tables.virtual_table n))
+                          ) (SSet.elements !(OcamlHashtbl.find tables.virtual_table n))
                         with Not_found -> []
                       end
                 )
               in
-              (*
-              debug "M-A-Same : %s produces (Debian) conflicts: %s" pkg.name (Printer.string_of_vpkglist l);
-              *)
-              let l' = add_arch_l native_arch arch (loadl tables l) in
-              (*
-              debug "M-A-Same : %s produces (CUDF) conflicts: %s" pkg.name (Cudf_types_pp.string_of_vpkglist l');
-              *)
-              l'
+              loadl ~native_arch ~package_arch:arch tables l
             )
       in
       multiarchconflicts @ multiarchconstraints
     in
     let _depends = 
-      List.map (add_arch_l ~deps:true native_arch pkgarch) 
-      (loadll tables (pkg.pre_depends @ pkg.depends))
+      loadll ~native_arch ~package_arch tables (pkg#pre_depends @ pkg#depends)
     in
     (* XXX: if ignore essential for the moment we also ignore keep *)
     let _keep =
       if options.ignore_essential then
         `Keep_none
-      else if (pkgarch <> native_arch && pkgarch <> "all") then
+      else if (package_arch <> native_arch && package_arch <> "all") then
         `Keep_none
       else
         set_keep pkg
@@ -530,7 +548,7 @@ let tocudf tables ?(options=default_options) ?(inst=false) pkg =
       Cudf.conflicts = _conflicts ;
       Cudf.provides = _provides ;
       Cudf.installed = add_inst inst pkg ;
-      Cudf.pkg_extra = add_extra options.extras_opt tables pkg ;
+      Cudf.pkg_extra = add_extra ~native_arch options.extras_opt tables pkg ;
     }
   end else
     (* :any and :native are not yet allowed in the Debian archive because of
@@ -540,31 +558,17 @@ let tocudf tables ?(options=default_options) ?(inst=false) pkg =
      * architecture are considered here anyways
      * XXX : in the future, dependencies on :$arch will be introduced (for example
      * for building cross compilers) they have to be handled here as well *)
-    (* XXX : hack I should look for : and not 3%a . encoding should be done
-     * after *)
-    let remove_qualifier = function
-      | n,v when String.ends_with n "%3anative" -> String.slice ~last:(-9) n, v
-      | n,v when String.ends_with n "%3aany" -> String.slice ~last:(-6) n, v
-      | n,v -> n, v
-    in
-    let _depends =
-      List.map (List.map remove_qualifier)
-      (loadll ~enc:true tables (pkg.pre_depends @ pkg.depends))
-    in
     { Cudf.default_package with
       Cudf.package = encpkgname ;
-      Cudf.version = get_cudf_version tables (pkg.name,pkg.version) ;
+      Cudf.version = get_cudf_version tables (pkg#name,pkg#version) ;
       Cudf.keep = if options.ignore_essential then `Keep_none else set_keep pkg;
-      Cudf.depends = _depends;
-      Cudf.conflicts = loadlc ~enc:true tables pkg.name (pkg.breaks @ pkg.conflicts) ;
-      Cudf.provides = loadlp ~enc:true tables pkg.provides ;
+      Cudf.depends = loadll tables (pkg#pre_depends @ pkg#depends); 
+      Cudf.conflicts = loadlc tables pkg#name (pkg#breaks @ pkg#conflicts) ;
+      Cudf.provides = loadlp tables pkg#provides ;
       Cudf.installed = add_inst inst pkg; 
       Cudf.pkg_extra = add_extra options.extras_opt tables pkg ;
     }
 
-let lltocudf = loadll
-let ltocudf = loadl
-
 let load_list ?options l =
   let timer = Util.Timer.create "Debian.ToCudf" in
   Util.Timer.start timer;
